import logging
from dataclasses import dataclass, field
from typing import Union, List

from obspy import Stream, read
from obspy.io.sac import SacIOError

from pylot.core.io.utils import validate_filenames
from pylot.core.util.dataprocessing import Metadata
from pylot.core.util.utils import get_stations, check_for_nan, check4rotated


@dataclass
class WaveformData:
    wfdata: Stream = field(default_factory=Stream)
    wforiginal: Union[Stream, None] = None
    wf_alt: Stream = field(default_factory=Stream)
    dirty: bool = False

    def load_waveforms(self, fnames: List[str], fnames_alt: List[str] = None, check_rotated=False, metadata=None, tstart=0, tstop=0):
        fn_list = validate_filenames(fnames)
        if not fn_list:
            logging.warning('No valid filenames given for loading waveforms')
        else:
            self.clear()
            self.add_waveforms(fn_list)

        if fnames_alt is None:
            pass
        else:
            alt_fn_list = validate_filenames(fnames_alt)
            if not alt_fn_list:
                logging.warning('No valid alternative filenames given for loading waveforms')
            else:
                self.add_waveforms(alt_fn_list, alternative=True)

        if not fn_list and not alt_fn_list:
            logging.error('No filenames or alternative filenames given for loading waveforms')
            return False

        self.merge()
        self.replace_nan()
        if not check_rotated or not metadata:
            pass
        else:
            self.rotate_zne()
        self.trim_station_traces()
        self.wforiginal = self.wfdata.copy()
        self.dirty = False
        return True

    def add_waveforms(self, fnames: List[str], alternative: bool = False):
        data_stream = self.wf_alt if alternative else self.wfdata
        warnmsg = ''
        for fname in set(fnames):
            try:
                data_stream += read(fname)
            except TypeError:
                try:
                    data_stream += read(fname, format='GSE2')
                except Exception as e:
                    try:
                        data_stream += read(fname, format='SEGY')
                    except Exception as e:
                        warnmsg += f'{fname}\n{e}\n'
            except SacIOError as se:
                warnmsg += f'{fname}\n{se}\n'

        if warnmsg:
            print(f'WARNING in add_waveforms: unable to read waveform data\n{warnmsg}')

    def clear(self):
        self.wfdata = Stream()
        self.wforiginal = None
        self.wf_alt = Stream()

    def reset(self):
        """
        Resets the waveform data to its original state.
        """
        if self.wforiginal:
            self.wfdata = self.wforiginal.copy()
        else:
            self.wfdata = Stream()
        self.dirty = False

    def merge(self):
        """
        check for gaps in Stream and merge if gaps are found
        """
        gaps = self.wfdata.get_gaps()
        if gaps:
            merged = ['{}.{}.{}.{}'.format(*gap[:4]) for gap in gaps]
            self.wfdata.merge(method=1)
            logging.info('Merged the following stations because of gaps:')
            for station in merged:
                logging.info(station)

    def replace_nan(self):
        """
        Replace all NaNs in data with 0. (in place)
        """
        self.wfdata = check_for_nan(self.wfdata)

    def rotate_zne(self, metadata: Metadata = None):
        """
        Check all traces in stream for rotation. If a trace is not in ZNE rotation (last symbol of channel code is numeric) and the trace
        is in the metadata with azimuth and dip, rotate it to classical ZNE orientation.
        Rotating the traces requires them to be of the same length, so, all traces will be trimmed to a common length as a
        side effect.
        """

        self.wfdata = check4rotated(self.wfdata, metadata)

    def trim_station_traces(self):
        """
        trim data stream to common time window
        """

        for station in get_stations(self.wfdata):
            station_traces = self.wfdata.select(station=station)
            station_traces.trim(starttime=max([trace.stats.starttime for trace in station_traces]),
                                endtime=min([trace.stats.endtime for trace in station_traces]))
